    jQuery(window).load(function () {
        JAK.SlideShow = JAK.ClassMaker.makeClass({ NAME:      'JAK.SlideShow',
                                                   VERSION:   '1.0',
                                                   CLASS:     'class',
                                                   IMPLEMENT: [JAK.ISignals] });

        JAK.SlideShow.prototype.$constructor = function (owner, name, settings) {
            this.owner = owner;
            this.options = { duration:       2,
                             autoplay:       false,
                             id:             false,
                             className:      'JAK-lightBox-image-browser-slideshow',
                             pauseId:        false,
                             pauseClassName: 'JAK-lightBox-image-browser-slideshow-pause',
                             playId:         false,
                             playClassName:  'JAK-lightBox-image-browser-slideshow-play' };

            for (var p in settings) {
                this.options[p] = settings[p];
            }

            this.interval = null;

            this._timeoutDone = this._timeoutDone.bind(this);

            this.dom = {};

            this._ev = [];

            this._addEvents();
            this._render();

            this.options.autoplay ? this._hidePlay() : this._hidePause();
        };

        JAK.SlideShow.prototype.$destructor = function () {
            this.removeListener('close', '_stop', this.owner);
            this.removeListener('transitionDone', '_next', this.owner);
            for (var i in this) {
                this[i] = null;
            }
        };

        JAK.SlideShow.prototype._addEvents = function () {
            if (this.options.autoplay) {
                this._ev['transitionDone'] = this.addListener('transitionDone', '_next', this.owner);
            }
            this._ev['close'] = this.addListener('close', '_stop', this.owner);
        };

        JAK.SlideShow.prototype._render = function () {
            var c = JAK.cel('div', this.options.className, this.options.id);

            var pause = JAK.cel('a', this.options.pauseClassName, this.options.pauseId);
            pause.href = '#';
            c.appendChild(pause);

            var play = JAK.cel('a', this.options.playClassName, this.options.playId);
            play.href = '#';
            c.appendChild(play);

            JAK.Events.addListener(pause, 'click', this, '_stopClick');
            JAK.Events.addListener(play, 'click', this, '_playClick');

            this.dom.pause = pause;
            this.dom.play = play;

            this.owner.dom.content.appendChild(c);
        };

        /**
        * metoda spuštěná při kliku na tlačítko pause
        * @param {event} e
        * @param {HTMLelement} elm
        */
        JAK.SlideShow.prototype._stopClick = function (e, elm) {
            JAK.Events.cancelDef(e);
            console.dir(this);
            this.removeListener(this._ev['transitionDone']);
            this._hidePause();
            this._stop();
        };

        /**
        * metoda spuštěná při kliku na tlačítko play
        * @param {event} e
        * @param {HTMLelement} elm
        */
        JAK.SlideShow.prototype._playClick = function (e, elm) {
            JAK.Events.cancelDef(e);
            this._ev['transitionDone'] = this.addListener('transitionDone', '_next', this.owner);
            this._hidePlay();
            this._next();
        };

        /**
        * zastavení slideshow
        */
        JAK.SlideShow.prototype._stop = function () {
            clearTimeout(this.interval);
        };

        /**
        * spuštění slideshow
        */
        JAK.SlideShow.prototype._next = function () {
            this.interval = setTimeout(this._timeoutDone, this.options.duration * 1000);
        };

        /**
        * schování tlačítka play, zobrazení pause
        */
        JAK.SlideShow.prototype._hidePlay = function () {
            this.dom.play.style.display = 'none';
            this.dom.pause.style.display = '';
        };

        /**
        * schování tlačítka pause, zobrazení play
        */
        JAK.SlideShow.prototype._hidePause = function () {
            this.dom.play.style.display = '';
            this.dom.pause.style.display = 'none';
        };

        /**
        * metoda volaná timeoutem, který je nastaven v metodě _next
        * zde je vlastní posunutí na další obrázek
        */
        JAK.SlideShow.prototype._timeoutDone = function () {
            clearTimeout(this.interval);
            this.owner.next();
        };
    });
