<?php /* хелпер шаблона  */

    class TemplateEmulator {

        // число демо фотографий товаров (для работы в демо режиме)
        protected $demo_mode_images_count = 30;

        // объект движка
        protected $cms = null;



        // ===================================================================
        /**
        *  Конструктор класса
        *
        *  -------------------------------------------------------------------
        *
        *  @access  public
        *  @param   object  $cms        объект движка
        *  @return  void
        */
        // ===================================================================

        public function __construct ( & $cms ) {
            $this->cms = & $cms;
            $this->setSmartyPlugins($cms);
        }



        // ===================================================================
        /**
        *  Подключение наших функций и модификаторов в шаблонизатор Smarty
        *
        *  -------------------------------------------------------------------
        *
        *  @access  protected
        *  @param   object  $cms        объект движка
        *  @return  void
        */
        // ===================================================================

        protected function setSmartyPlugins ( & $cms ) {

            // регистрируем шаблонизационную функцию idProducts
            $cms->smarty->registerPlugin('function', 'idProducts', array($this, 'idProducts'));

            // регистрируем шаблонизационную функцию fixDelivery
            $cms->smarty->registerPlugin('function', 'fixDelivery', array($this, 'fixDelivery'));

            // регистрируем шаблонизационную функцию catalogTags
            $cms->smarty->registerPlugin('function', 'catalogTags', array($this, 'catalogTags'));

            // регистрируем шаблонизационную функцию shopTweets
            $cms->smarty->registerPlugin('function', 'shopTweets', array($this, 'shopTweets'));
        }



        // ===================================================================
        /**
        *  Обработчик шаблонизационной функции idProducts
        *
        *  Действие: извлекает из базы данных товары с перечисленными
        *  идентификаторами и помещает записи в указанную smarty-переменную
        *  var, чтобы эти записи стали доступны внутри шаблона.
        *
        *  -------------------------------------------------------------------
        *
        *  @access  public
        *  @param   array   $params     массив параметров, указанных в вызове:
        *                                   ['var'] = в какую переменную возвратить результат
        *                                   ['ids'] = список идентификаторов товаров
        *  @param   object  $smarty     объект шаблонизатора Smarty
        *  @return  string              всегда пустая строка
        */
        // ===================================================================

        public function idProducts ( $params = null, & $smarty = null ) {

            // проверяем наличие обязательного параметра
            $field = 'var';
            if (isset($params[$field]) && is_string($params[$field]) && $params[$field] != '') {

                // список идентификаторов
                $items = 'ids';
                $items = isset($params[$items]) && is_array($params[$items]) ? $params[$items] : array();
                $ids = '';
                foreach ($items as $id) {
                    if (is_numeric($id) && !empty($id)) {
                        if ($ids != '') $ids .= ',';
                        $ids .= preg_replace('/[\'`"]/', '', $id);
                    }
                }
                if ($ids != '') {

                    // текущий пользователь, если есть
                    $user = null;
                    if (isset($this->cms->user)) $user = & $this->cms->user;

                    // настраиваем фильтр записей
                    $filter = new stdClass;

                        // только товары с такими ИДами
                        $filter->ids = $ids;

                        // только разрешенные к показу
                        $filter->enabled = 1;

                        // если это обычный посетитель, тогда только не скрытые от неавторизованных
                        if (!isset($user->user_id)) $filter->hidden = 0;

                        // упорядочить список по категориям
                        $filter->sort = SORT_PRODUCTS_MODE_BY_CATEGORY;
                        $filter->sort_direction = SORT_DIRECTION_ASCENDING;

                        // процент скидки пользователя и его ценовая группа
                        $filter->discount = isset($user->discount) ? $user->discount : 0;
                        $filter->price_id = isset($user->price_id) ? $user->price_id : 0;

                        // просим дочитать связанные товары
                        $filter->with_related = 1;

                    // читаем список товаров (здесь заключаем весь процесс чтения
                    // в команды трассировщика в целях поддержки отладки, но вы можете
                    // убрать 2 крайние команды, если вас это не интересует)
                    $this->cms->db->open_tracing_method('SMARTY idProducts');
                        $items = null;
                        $this->cms->db->products->get($items, $filter);
                        $this->cms->db->products->unpackRecords($items, $filter);
                    $this->cms->db->close_tracing_method();

                    // передаем в указанную переменную
                    if (!is_object($smarty)) return $items;
                    $smarty->assignByRef($params[$field], $items);
                }
            }
            return '';
        }



        // ===================================================================
        /**
        *  Обработчик шаблонизационной функции fixDelivery
        *
        *  Действие: корректирует запись о способе доставки, если не была
        *  откорректирована ранее.
        *
        *  -------------------------------------------------------------------
        *
        *  @access  public
        *  @param   array   $params     массив параметров, указанных в вызове:
        *                                   ['item'] = запись о способе доставки
        *                                   ['var'] = в какую переменную возвратить результат
        *  @param   object  $smarty     объект шаблонизатора Smarty
        *  @return  string              всегда пустая строка
        */
        // ===================================================================

        public function fixDelivery ( $params = null, & $smarty = null ) {

            // проверяем наличие обязательного параметра
            $field = 'var';
            if (isset($params[$field]) && is_string($params[$field]) && $params[$field] != '') {

                // ищем вариант и поле цены
                $item = 'item';
                if (isset($params[$item])) {
                    if (isset($params[$item]->types_ids) && is_string($params[$item]->types_ids)) {
                        $item = $params[$item];
                        $this->cms->db->fix_deliveries_record($item);
                    }

                    // передаем в указанную переменную
                    if (!is_object($smarty)) return $item;
                    $smarty->assignByRef($params[$field], $item);
                }
            }
            return '';
        }



        // ===================================================================
        /**
        *  Обработчик шаблонизационной функции catalogTags
        *
        *  Действие: извлекает уникальный список тегов товаров и помещает как
        *  массив строк в smarty-переменную var, чтобы эти записи стали
        *  доступны внутри шаблона.
        *
        *  -------------------------------------------------------------------
        *
        *  @access  public
        *  @param   array   $params     массив параметров, указанных в вызове:
        *                                   ['var'] = в какую переменную возвратить результат
        *  @param   object  $smarty     объект шаблонизатора Smarty
        *  @return  string              всегда пустая строка
        */
        // ===================================================================

        public function catalogTags ( $params = null, & $smarty = null ) {

            // проверяем наличие обязательного параметра
            $field = 'var';
            if (isset($params[$field]) && is_string($params[$field]) && $params[$field] != '') {

                // текущий пользователь, если есть
                $user = null;
                if (isset($this->cms->user)) $user = & $this->cms->user;

                // читаем список тегов (здесь заключаем весь процесс чтения
                // в команды трассировщика в целях поддержки отладки, но вы можете
                // убрать 2 крайние команды, если вас это не интересует)
                $this->cms->db->open_tracing_method('SMARTY catalogTags');
                    $items = array();
                    $query = 'SELECT `products`.`tags` '
                           . 'FROM `products` '
                           . 'WHERE `products`.`enabled` = 1 '
                                 . (!isset($user->user_id) ? 'AND `products`.`hidden` = 0 ' : '') . ';';
                    $result = $this->cms->db->query($query);
                    if ($result !== FALSE) {
                        $this->cms->db->open_tracing_method('SMARTY catalogTags [parsing]');
                            while ($row = $this->cms->db->fetch_object($result)) {
                                $row = preg_replace('/[\s\t\r\n]+/', ' ', $row->tags);
                                $row = preg_replace('/[^а-яёa-z0-9,\s\-]+/iu', ',', $row);
                                $row = explode(',', $row);
                                foreach ($row as $tag) {
                                    $tag = trim($tag, ' -');
                                    if ($tag != '') {
                                        $tag = preg_replace('/[\s-]+/', ' ', $tag);
                                        $query = strtolower($tag);
                                        if (!isset($items[$query])) $items[$query] = $tag;
                                    }
                                }
                            }
                            ksort($items, SORT_STRING);
                        $this->cms->db->close_tracing_method();
                    }
                $this->cms->db->close_tracing_method();

                // передаем в указанную переменную
                if (!is_object($smarty)) return $items;
                $smarty->assignByRef($params[$field], $items);
            }
            return '';
        }



        // ===================================================================
        /**
        *  Обработчик шаблонизационной функции shopTweets
        *
        *  Действие: извлекает из желаемой текстовой базы данных count "твитов",
        *  расположенных на воображаемой странице под номером page, и помещает
        *  их записи в указанную smarty-переменную var, чтобы стали доступны
        *  внутри шаблона.
        *
        *  -------------------------------------------------------------------
        *
        *  @access  public
        *  @param   array   $params     массив параметров, указанных в вызове:
        *                                   ['var'] = в какую переменную возвратить результат
        *                                             дополнительно будут созданы:
        *                                                 ..._frows = общее число таких записей
        *                                                 ..._arows = число записей на текущей странице
        *                                   ['count'] = число записей на странице (по умолчанию 32)
        *                                   ['page'] = номер текущей страницы (по умолчанию 1)
        *                                   ['topic'] = имя желаемого топика, условно это имя базы данных (по умолчанию main)
        *  @param   object  $smarty     объект шаблонизатора Smarty
        *  @return  string              всегда пустая строка
        */
        // ===================================================================

        public function shopTweets ( $params = null, & $smarty = null ) {

            // проверяем наличие обязательного параметра
            $field = 'var';
            if (isset($params[$field]) && is_string($params[$field]) && $params[$field] != '') {

                // опциональные параметры
                $count = 'count';
                $count = isset($params[$count]) ? max(1, @ intval($params[$count])) : 32;

                $page = 'page';
                $page = isset($params[$page]) ? max(1, @ intval($params[$page])) : 1;

                $topic = 'topic';
                $topic = isset($params[$topic]) && is_string($params[$topic]) ? $params[$topic] : 'main';



                // проверяем наличие базы (облегченно, поскольку могут именовать полем URL)
                $items = array();
                $module = 'ShopTweets';
                $descriptor = array('dbname'  => $topic,
                                    'folder'  => $module,
                                    'columns' => array('enabled', 'id', 'created', 'modified', 'title', 'text', 'url', 'image', 'ip'),
                                    'easy'    => TRUE);
                if ($this->cms->textdb->check($descriptor)) {

                    // если это админ запостил твит (иначе игнорируем без сообщения ошибки)
                    if ($this->cms->textdb->checkAdminRights($module)) {
                        $fields = $this->cms->request->getRequest($module);
                        if (!empty($fields['post'])) {
                            $fields['ip'] = $this->cms->security->getVisitorIp();

                            // если твит корректируют или удаляют
                            if (isset($fields['id']) && trim($fields['id']) != '') {

                                // если удаляют
                                if (!empty($fields['delete'])) {
                                    $this->cms->textdb->delete($descriptor, $fields);

                                // иначе если корректируют
                                } elseif (isset($fields['enabled']) || isset($fields['title']) || isset($fields['text']) || isset($fields['url']) || isset($fields['image'])) {

                                    // обезопасим полученные сведения
                                    if (isset($fields['title'])) $fields['title'] = $this->cms->textdb->asPlainText($fields['title']);
                                    if (isset($fields['text'])) $fields['text'] = $this->cms->textdb->asPlainText($fields['text']);
                                    if (isset($fields['image'])) $fields['image'] = $this->cms->textdb->asUrl($fields['image']);
                                    if (isset($fields['url'])) $fields['url'] = $this->cms->textdb->asUrl($fields['url']);
                                    if (isset($fields['enabled'])) $fields['enabled'] = $fields['enabled'] ? 1 : 0;
                                    if (isset($fields['created'])) unset($fields['created']);

                                    // исправляем твит
                                    $fields['id'] = trim($fields['id']);
                                    $fields['modified'] = $this->cms->textdb->datetimeMarker();
                                    $this->cms->textdb->update($descriptor, $fields);
                                }
                            } else {

                                // обезопасим полученные сведения
                                $fields['title'] = isset($fields['title']) ? $this->cms->textdb->asPlainText($fields['title']) : '';
                                $fields['text'] = isset($fields['text']) ? $this->cms->textdb->asPlainText($fields['text']) : '';
                                $fields['image'] = isset($fields['image']) ? $this->cms->textdb->asUrl($fields['image']) : '';
                                $fields['url'] = isset($fields['url']) ? $this->cms->textdb->asUrl($fields['url']) : '';
                                if (isset($fields['modified'])) unset($fields['modified']);
                                if (isset($fields['id'])) unset($fields['id']);

                                // если обязательное заполнено, добавляем твит
                                if (!empty($fields['title']) || !empty($fields['text']) || !empty($fields['image'])) {
                                    $fields['enabled'] = 1;
                                    $fields['created'] = $this->cms->textdb->datetimeMarker();
                                    $this->cms->textdb->add($descriptor, $fields);
                                }
                            }
                        }
                    }

                    // читаем список твитов
                    $start = ($page - 1) * $count;
                    $filter = array('enabled' => 1);
                    $items = $this->cms->textdb->get($descriptor, $filter, $start, $count);
                }



                // передаем в указанную переменную
                if (!is_object($smarty)) return $items;
                $smarty->assignByRef($params[$field], $items);
                $smarty->assign($params[$field] . '_frows', $this->cms->textdb->found_rows);
                $smarty->assign($params[$field] . '_arows', $this->cms->textdb->affected_rows);
            }
            return '';
        }
    }

    return;
?>